/*
 * XmlInsert.java
 * :tabSize=8:indentSize=8:noTabs=false:
 * :folding=explicit:collapseFolds=1:
 *
 * Copyright (C) 2001, 2002 Slava Pestov
 *
 * The XML plugin is licensed under the GNU General Public License, with
 * the following exception:
 *
 * "Permission is granted to link this code with software released under
 * the Apache license version 1.1, for example used by the Xerces XML
 * parser package."
 */

package xml;

//{{{ Imports
import javax.swing.event.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.AWTEventMulticaster;  // this for MyInputMethodListener
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.util.List;
import java.awt.Color;
import javax.swing.border.BevelBorder;
import org.gjt.sp.jedit.msg.*;
import org.gjt.sp.jedit.textarea.*;
import org.gjt.sp.jedit.*;
import sidekick.*;
import sidekick.GhostElement;
import xml.completion.*;

// JL: added
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.tree.*;
import java.util.Enumeration;
//}}}

public class XmlInsert extends JPanel implements EBComponent
{
	//{{{ XmlInsert constructor
	public XmlInsert(View view, boolean sideBySide)
	{
		this.view = view;

		setLayout(new GridLayout(sideBySide ? 1 : 3,
			sideBySide ? 3 : 1,3,3));

		elementPanel = new JPanel(new BorderLayout());
                JLabelElementPanel = new JLabel(jEdit.getProperty("xml-insert.elements"));
		elementPanel.add(BorderLayout.NORTH, JLabelElementPanel);

		elementList = new JList();
		elementList.setCellRenderer(new XmlListCellRenderer());
		elementList.addMouseListener(aMouseHandler);
		elementList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
                elementList.addKeyListener(aKeyHandler);
		elementPanel.add(BorderLayout.CENTER, new JScrollPane(elementList));

               //--
               //17.5.2004 OP: this is for adding elements in well-formed document
               elementAdd = new JTextField();
               elementAdd.setBorder(new BevelBorder(BevelBorder.LOWERED));

               JButtonAdd = new JButton(jEdit.getProperty("xml-insert.addButton"));
               JButtonAdd.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent e) {
                   insertElement(getelementAddText());
                 }
               });

               JPanelElementAdd = new JPanel(new BorderLayout());
               JPanelElementAdd.add(elementAdd, BorderLayout.CENTER);
               JPanelElementAdd.add(JButtonAdd, BorderLayout.EAST);



               elementPanel.add(BorderLayout.SOUTH,JPanelElementAdd);

	       //--

               add(elementPanel);

		JPanel entityPanel = new JPanel(new BorderLayout());
		entityPanel.add(BorderLayout.NORTH,
			new JLabel(jEdit.getProperty("xml-insert.entities")));

		entityList = new JList();
		entityList.setCellRenderer(new XmlListCellRenderer());
		entityList.addMouseListener(new MouseHandler());
		entityList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		entityPanel.add(BorderLayout.CENTER,
			new JScrollPane(entityList));

		add(entityPanel);

		JPanel idPanel = new JPanel(new BorderLayout());
		idPanel.add(BorderLayout.NORTH,
			new JLabel(jEdit.getProperty("xml-insert.ids")));

		idList = new JList();
		idList.setCellRenderer(new XmlListCellRenderer());
		idList.addMouseListener(new MouseHandler());
		idList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		idPanel.add(BorderLayout.CENTER,new JScrollPane(idList));

		add(idPanel);

		caretHandler = new CaretHandler();

                // OP: these attributes inform the place where element should be added
                offsetStart = -1;
                offsetEnd = -1;

		update();

		updateTimer = new Timer(0,new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				updateTagList();
			}
		});
	} //}}}

        private void insertElement(String pElementName) {
         EditPane editPane = view.getEditPane();
         JEditTextArea textArea = editPane.getTextArea();
         Buffer buffer = editPane.getBuffer();

         String openingTag = "<" + pElementName + ">";
         String closingTag = "</" + pElementName + ">";

         Selection[] selection = textArea.getSelection();

         if(selection.length > 0)
         {
                 try
                 {
                         buffer.beginCompoundEdit();

                         for(int i = 0; i < selection.length; i++)
                         {
                                 buffer.insert(selection[i].getStart(),
                                         openingTag);
                                 buffer.insert(selection[i].getEnd(),
                                         closingTag);
                         }
                 }
                 finally
                 {
                         buffer.endCompoundEdit();
                 }

                 textArea.selectNone();
         }
         else
         {
                 buffer.insert(offsetStart,openingTag);
                 buffer.insert(openingTag.length()+offsetEnd,closingTag);
         }

         textArea.selectNone();
         textArea.requestFocus();
         disableElementPanel();
        }

        private String getelementAddText() {
          return elementAdd.getText();
        }

        public void enableElementPanel() {
          //if (elementList.getMouseListeners().length == 0) elementList.addMouseListener(aMouseHandler);
          if (elementList.getKeyListeners().length == 0) elementList.addKeyListener(aKeyHandler);
          elementList.setEnabled(true);
          JButtonAdd.setEnabled(true);
          elementAdd.setEnabled(true);

        }

        private void disableElementPanel() {
          //elementList.removeMouseListener(aMouseHandler);
          elementList.removeKeyListener(aKeyHandler);
          elementList.setEnabled(false);
          JButtonAdd.setEnabled(false);
          elementAdd.setEnabled(false);
        }


	//{{{ addNotify() method
	public void addNotify()
	{
		super.addNotify();
		EditBus.addToBus(this);

		EditPane[] editPanes = view.getEditPanes();
		for(int i = 0; i < editPanes.length; i++)
		{
			JEditTextArea textArea = editPanes[i].getTextArea();
			textArea.addCaretListener(caretHandler);
		}
	} //}}}

	//{{{ removeNotify() method
	public void removeNotify()
	{
		super.removeNotify();
		EditBus.removeFromBus(this);

		EditPane[] editPanes = view.getEditPanes();
		for(int i = 0; i < editPanes.length; i++)
		{
			JEditTextArea textArea = editPanes[i].getTextArea();
			textArea.removeCaretListener(caretHandler);
		}

		updateTimer.stop();
	} //}}}

	//{{{ handleMessage() method
	public void handleMessage(EBMessage msg)
	{
		//{{{ EditPaneUpdate
		if(msg instanceof EditPaneUpdate)
		{
			EditPaneUpdate emsg = (EditPaneUpdate)msg;
			EditPane editPane = emsg.getEditPane();
			if(emsg.getWhat() == EditPaneUpdate.BUFFER_CHANGED
				&& editPane == view.getEditPane())
			{
                               offsetStart = -1;
                               offsetEnd = -1;
				update();
			}
			else if(emsg.getWhat() == EditPaneUpdate.CREATED
				&& editPane.getView() == view)
			{
				JEditTextArea textArea = editPane.getTextArea();
				textArea.addCaretListener(caretHandler);
			}
			else if(emsg.getWhat() == EditPaneUpdate.DESTROYED
				&& editPane.getView() == view)
			{
				JEditTextArea textArea = editPane.getTextArea();
				textArea.removeCaretListener(caretHandler);
			}
			else if(emsg.getWhat() == EditPaneUpdate.BUFFER_CHANGED) {
                          offsetStart = -1;
                          offsetEnd = -1;
                          update();
                        }
		} //}}}
		//{{{ BufferUpdate
		else if(msg instanceof BufferUpdate)
		{
			BufferUpdate bmsg = (BufferUpdate)msg;
			if((bmsg.getWhat() == BufferUpdate.PROPERTIES_CHANGED
				|| bmsg.getWhat() == BufferUpdate.LOADED)
				&& bmsg.getBuffer() == view.getBuffer())
			{
				update();
			}
		} //}}}
		//{{{ SideKickUpdate
		else if(msg instanceof SideKickUpdate)
		{
			if(((SideKickUpdate)msg).getView() == view) {
                          update();
                          enableElementPanel();
                        }
		} //}}}
                else if(msg instanceof UpdateTagListMsg) {
                  offsetStart = ((UpdateTagListMsg)msg).getOffsetStart();
                  offsetEnd = ((UpdateTagListMsg)msg).getOffsetEnd();
                  update();
                }

                else if(msg instanceof ShowEditTagDialog) {
                  view.getTextArea().setCaretPosition(view.getTextArea().getCaretPosition()+1);
                  XmlActions.showEditTagDialog(view);
                }

	} //}}}

	//{{{ Private members

	//{{{ Instance variables
	private View view;
	private CaretHandler caretHandler;
	private List elements;
	private JList elementList;
	private JList entityList;
	private JList idList;
        private JPanel elementPanel;
        private JLabel JLabelElementPanel;
        private JTextField elementAdd;
        private JPanel JPanelElementAdd;
        private JButton JButtonAdd;
	private boolean html;

        private MouseHandler aMouseHandler = new MouseHandler();
        private KeyHandler aKeyHandler = new KeyHandler();

	private int delay;
	private Timer updateTimer;

        private int offsetStart;
        private int offsetEnd;
	//}}}

	//{{{ update() method
	private void update()
	{
		SideKickParsedData _data = SideKickParsedData.getParsedData(view);
		if(!(_data instanceof XmlParsedData))
		{
			setDeclaredEntities(null);
			setDeclaredIDs(null);
			html = false;
		}
		else
		{
			XmlParsedData data = (XmlParsedData)_data;
			setDeclaredEntities(data.getNoNamespaceCompletionInfo().entities);
			setDeclaredIDs(data.ids);
			html = data.html;
		}

		updateTagList();
	} //}}}

	//{{{ showNotParsedMessage() method
	private void showNotParsedMessage()
	{
		setDeclaredElements(null);
		setDeclaredEntities(null);
		setDeclaredIDs(null);
	} //}}}

	//{{{ setDeclaredElements() method
	private void setDeclaredElements(List elements)
	{
                this.elements = elements;

		if(elements == null || elements.size() == 0)
		{
			DefaultListModel model = new DefaultListModel();
			model.addElement(jEdit.getProperty("xml-insert.not-parsed"));
			elementList.setModel(model);
		}
		else
		{
                        if (elements.get(0) instanceof NotValidElementDecl)  {
                          JLabelElementPanel.setText(jEdit.getProperty("xml-insert.elementsnotvalid"));
                          JPanelElementAdd.setVisible(true);

                        } else if (elements.get(0) instanceof ElementDecl) {
                          JLabelElementPanel.setText(jEdit.getProperty("xml-insert.elements"));
                          JPanelElementAdd.setVisible(false);
                        }
			ArrayListModel model = new ArrayListModel(elements);
			elementList.setModel(model);

			if(model.getSize() != 0)
			{
				Rectangle cellBounds = elementList
					.getCellBounds(0,0);
				if(cellBounds != null)
				{
					elementList.setFixedCellHeight(cellBounds.height);
				}
			}
		}
	} //}}}

	//{{{ setDeclaredEntities() method
	private void setDeclaredEntities(List entities)
	{
		if(entities == null)
		{
			DefaultListModel model = new DefaultListModel();
			model.addElement(jEdit.getProperty("xml-insert.not-parsed"));
			entityList.setModel(model);
		}
		else
		{
			ArrayListModel model = new ArrayListModel(entities);
			entityList.setModel(model);

			if(model.getSize() != 0)
			{
				Rectangle cellBounds = entityList
					.getCellBounds(0,0);
				if(cellBounds != null)
				{
					entityList.setFixedCellHeight(cellBounds.height);
				}
			}
		}

	} //}}}

	//{{{ setDeclaredIDs() method
	private void setDeclaredIDs(List ids)
	{
		if(ids == null)
		{
			DefaultListModel model = new DefaultListModel();
			model.addElement(jEdit.getProperty("xml-insert.not-parsed"));
			idList.setModel(model);
		}
		else
		{
			ArrayListModel model = new ArrayListModel(ids);
			idList.setModel(model);

			if(model.getSize() != 0)
			{
				Rectangle cellBounds = idList.getCellBounds(0,0);
				if(cellBounds != null)
				{
					idList.setFixedCellHeight(cellBounds.height);
				}
			}
		}

	} //}}}

	//{{{ updateTagList() method
        /**
         * 19.05.2004 JL: modified. added the case if no dtd
         */
        private void updateTagList()
	{
               //System.err.print("Tassa tullaan updateTaglist metodiin");

               // OP: if this is true clears the elementlist
               if (offsetStart < 0) {
                 setDeclaredElements(null);
                 return;
               }

		Buffer buffer = view.getBuffer();

		// OP: Here we check which element is selected from the
                //    tree and if it is GhostElement then we update the
                //    tag list. If its not then clear the tag list

               SideKickParsedData _data = SideKickParsedData.getParsedData(view);

               String bufferText = buffer.getText(0, buffer.getLength());
               boolean hasDTD = (bufferText.lastIndexOf("<!DOCTYPE") != -1); // what if <!DOCTYPE is somewhere else...


		if(_data instanceof XmlParsedData ) {
                  if ( hasDTD ) {
                    XmlParsedData data = (XmlParsedData) _data;
                    setDeclaredElements(data.getAllowedElements(
                        buffer, offsetStart));
                  } else { // data.hasDTD() returns false
                    ArrayList treeElements = new ArrayList();

                    JTree tree = ((SideKickTree)(view.getDockableWindowManager().getDockableWindow("sidekick-tree"))).getTree();


                    String name = "";
                    TreeNode root = (TreeNode)tree.getModel().getRoot();


                    // Traverse tree from root adding treepaths for all nodes to list
                    getPaths(new TreePath(root), treeElements);

                    ArrayList elementNames = new ArrayList();
    		    for(int i = 0; i < treeElements.size(); i++) {
                       DefaultMutableTreeNode node = (DefaultMutableTreeNode)(((TreePath)(treeElements.get(i))).getLastPathComponent());
                       Object userObj = node.getUserObject();
                       if ( !(userObj instanceof sidekick.Asset) ) continue;
                       name = ((sidekick.Asset)userObj).name;

                      if (!elementNames.contains(name)) {
                        elementNames.add(name);
                      }
                    }

                    // sort in alphabetical order
                    java.util.Collections.sort(elementNames,new org.gjt.sp.jedit.MiscUtilities.Compare() {
                        public int compare(Object obj1, Object obj2) {
                          return MiscUtilities.compareStrings((String)obj1,(String)obj2,true);
                       }
                    }
);

                    ArrayList finalList = new ArrayList();
                    for(int i = 0; i < elementNames.size(); i++) {
                      name = (String)elementNames.get(i);
                      finalList.add(new NotValidElementDecl(name));
                    }

                    setDeclaredElements(finalList); // all different elements found from document
                  }
                } // not instanceof XmlParsedData
		else
		{
			setDeclaredElements(null);
		}
	} //}}}

   private void getPaths(TreePath parent, List list) {
     // Add node to list
     list.add(parent);

     // Create paths for all children
     TreeNode node = (TreeNode) parent.getLastPathComponent();
     if (node.getChildCount() >= 0) {
       for (Enumeration e = node.children(); e.hasMoreElements(); ) {
         TreeNode n = (TreeNode) e.nextElement();
         TreePath path = parent.pathByAddingChild(n);
         getPaths(path, list);
       }
     }
   }



	//{{{ updateTagListWithDelay() method
	private void updateTagListWithDelay()
	{
		if(updateTimer.isRunning())
			updateTimer.stop();

		updateTimer.setInitialDelay(200);
		updateTimer.setRepeats(false);
		updateTimer.start();
	} //}}}

	//}}}


       //{{{ ArrayListModel class
	static class ArrayListModel implements ListModel
	{
		List list;

		ArrayListModel(List list)
		{
			this.list = list;
		}

		public int getSize()
		{
			return list.size();
		}
		public Object getElementAt(int index)
		{
			return list.get(index);
		}

		public void addListDataListener(ListDataListener l) {}
		public void removeListDataListener(ListDataListener l) {}
	} //}}}

	//{{{ CaretHandler class
	class CaretHandler implements CaretListener
	{
		public void caretUpdate(CaretEvent evt)
		{
			if(evt.getSource() == view.getTextArea())
				updateTagListWithDelay();
		}
	} //}}}


              /**
               *  14.05.2004
               *     JL: Added KeyHandler so we can add elements by pressing enter
               *      when the desired element is selected.
               *
               *    19.05.2004 JL: Added the case if (obj instanceof NotValidElementDecl)
               */
              class KeyHandler extends KeyAdapter
              {
                      public void keyPressed(KeyEvent evt)
                      {
                        if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
                          evt.consume();

                          Object obj = elementList.getModel().getElementAt(elementList.getSelectedIndex());
                          if ( (obj instanceof ElementDecl) ) {
                            ElementDecl element = (ElementDecl) obj;

                            // show edit tag dialog box
                            XmlActions.showEditTagDialog(view, element, offsetStart, offsetEnd);
                          } else if (obj instanceof NotValidElementDecl) {
                            EditPane editPane = view.getEditPane();
                            JEditTextArea textArea = editPane.getTextArea();
                            Buffer buffer = editPane.getBuffer();

                            NotValidElementDecl element = (NotValidElementDecl)obj;
                            String openingTag = "<" + element.elementName + ">";
                            String closingTag = "</" + element.elementName + ">";

                            Selection[] selection = textArea.getSelection();

                            if(selection.length > 0)
                            {
                                    try
                                    {
                                            buffer.beginCompoundEdit();

                                            for(int i = 0; i < selection.length; i++)
                                            {
                                                    buffer.insert(selection[i].getStart(),
                                                            openingTag);
                                                    buffer.insert(selection[i].getEnd(),
                                                            closingTag);
                                            }
                                    }
                                    finally
                                    {
                                            buffer.endCompoundEdit();
                                    }

                                    textArea.selectNone();
                            }
                            else
                            {

                                    // OP: this adds the element to the place
                                    //     that the ghostelement shows
                                    buffer.insert(offsetStart,openingTag);
                                    buffer.insert(openingTag.length()+offsetEnd,closingTag);
                            }

                            textArea.selectNone();
                            textArea.requestFocus();
                            disableElementPanel();
                          }
                        }
                      }
              } //}}}


              //{{{ MouseHandler class
              class MouseHandler extends MouseAdapter
              {
                      public void mouseClicked(MouseEvent evt)
                      {
                              //{{{ Handle clicks in element list
                              if(evt.getSource() == elementList)
                              {
                                      int index = elementList.locationToIndex(
                                              evt.getPoint());
                                      if(index == -1)
                                              return;

                                      idList.setSelectedIndex(index);
                                      Object obj = elementList.getModel().getElementAt(index);
                                      if (obj instanceof NotValidElementDecl) {
                                        EditPane editPane = view.getEditPane();
                                        JEditTextArea textArea = editPane.getTextArea();
                                        Buffer buffer = editPane.getBuffer();

                                        NotValidElementDecl element = (NotValidElementDecl)obj;
                                        String openingTag = "<" + element.elementName + ">";
                                        String closingTag = "</" + element.elementName + ">";

                                        Selection[] selection = textArea.getSelection();

                                        if(selection.length > 0)
                                        {
                                                try
                                                {
                                                        buffer.beginCompoundEdit();

                                                        for(int i = 0; i < selection.length; i++)
                                                        {
                                                                buffer.insert(selection[i].getStart(),
                                                                        openingTag);
                                                                buffer.insert(selection[i].getEnd(),
                                                                        closingTag);
                                                        }
                                                }
                                                finally
                                                {
                                                        buffer.endCompoundEdit();
                                                }

                                                textArea.selectNone();
                                        }
                                        else
                                        {
                                                if ( textArea.isFocusOwner() ) {
                                                  textArea.setSelectedText(openingTag);
                                                  int caret = textArea.getCaretPosition();
                                                  textArea.setSelectedText(closingTag);
                                                  textArea.setCaretPosition(caret);
                                                }

                                                SideKickTree tree = (SideKickTree)view.getDockableWindowManager().getDockableWindow("sidekick-tree");

                                                // OP: this adds the element to the place
                                                //     that the ghostelement shows

                                                if (tree.contentAreaHasFocus()) {
                                                  buffer.insert(offsetStart, openingTag);
                                                  buffer.insert(openingTag.length() + offsetEnd, closingTag);
                                                }

                                                if (tree.treeHasFocus()) {
                                                    buffer.insert(offsetStart, openingTag);
                                                    buffer.insert(openingTag.length() + offsetEnd, closingTag);
                                                }

					  }

                                        textArea.selectNone();
                                        textArea.requestFocus();
                                        disableElementPanel();
                                        return;
                                      }

                                      if(!(obj instanceof ElementDecl))
                                              return;

                                      ElementDecl element = (ElementDecl)obj;

                                      EditPane editPane = view.getEditPane();
                                      JEditTextArea textArea = editPane.getTextArea();
                                      Buffer buffer = editPane.getBuffer();

                                      //{{{ Handle right mouse button click
                                      if(GUIUtilities.isPopupTrigger(evt))
                                      {
                                              String openingTag = "<" + element.name
                                                      + element.getRequiredAttributesString()
                                                      + (element.empty && !html
                                                      ? XmlActions.getStandaloneEnd() : ">");
                                              String closingTag;
                                              if(element.empty)
                                                      closingTag = "";
                                              else
                                                      closingTag = "</" + element.name + ">";

                                              Selection[] selection = textArea.getSelection();

                                              if(selection.length > 0)
                                              {
                                                      try
                                                      {
                                                              buffer.beginCompoundEdit();

                                                              for(int i = 0; i < selection.length; i++)
                                                              {
                                                                      buffer.insert(selection[i].getStart(),
                                                                              openingTag);
                                                                      buffer.insert(selection[i].getEnd(),
                                                                              closingTag);
                                                              }
                                                      }
                                                      finally
                                                      {
                                                              buffer.endCompoundEdit();
                                                      }

                                                      textArea.selectNone();
                                              }
                                              else
                                              {
                                                if ( textArea.isFocusOwner() ) {
                                                  textArea.setSelectedText(openingTag);
                                                  int caret = textArea.getCaretPosition();
                                                  textArea.setSelectedText(closingTag);
                                                  textArea.setCaretPosition(caret);
                                                }

                                                SideKickTree tree = (SideKickTree)view.getDockableWindowManager().getDockableWindow("sidekick-tree");

                                                // OP: this adds the element to the place
                                                //     that the ghostelement shows

                                                if (tree.contentAreaHasFocus()) {
                                                  buffer.insert(offsetStart, openingTag);
                                                  buffer.insert(openingTag.length() + offsetEnd, closingTag);
                                                }

                                                if (tree.treeHasFocus()) {
                                                    buffer.insert(offsetStart, openingTag);
                                                    buffer.insert(openingTag.length() + offsetEnd, closingTag);
                                                }

                                              }

                                              textArea.selectNone();
                                              textArea.requestFocus();
                                      } //}}}
                                      else
                                      {
                                              // show edit tag dialog box
                                              XmlActions.showEditTagDialog(view,element,offsetStart,offsetEnd);
                                      }
                                      disableElementPanel();
                              } //}}}
                              //{{{ Handle clicks in entity list
                              else if(evt.getSource() == entityList)
                              {
                                      int index = entityList.locationToIndex(
                                              evt.getPoint());
                                      if(index == -1)
                                              return;

                                      Object obj = entityList.getModel().getElementAt(index);
                                      if(!(obj instanceof EntityDecl))
                                              return;

                                      EntityDecl entity = (EntityDecl)obj;

                                      JEditTextArea textArea = view.getTextArea();

                                      textArea.setSelectedText("&" + entity.name + ";");
                                      textArea.requestFocus();
                              } //}}}
                              //{{{ Handle clicks in ID list
                              else if(evt.getSource() == idList)
                              {
                                      int index = idList.locationToIndex(
                                              evt.getPoint());
                                      if(index == -1)
                                              return;

                                      idList.setSelectedIndex(index);
                                      Object obj = idList.getModel().getElementAt(index);
                                      if(!(obj instanceof IDDecl))
                                              return;

                                      IDDecl id = (IDDecl)obj;

                                      JEditTextArea textArea = view.getTextArea();

                                      if(GUIUtilities.isPopupTrigger(evt))
                                      {
                                              Buffer buffer = jEdit.openFile(view,id.uri);
                                              if(buffer == null)
                                                      return;
                                              int line = Math.min(buffer.getLineCount() - 1,id.line);
                                              int column = Math.min(buffer.getLineLength(line),id.column);
                                              int offset = buffer.getLineStartOffset(line) + column;
                                              textArea.setCaretPosition(offset);
                                      }
                                      else
                                      {
                                              textArea.setSelectedText(id.id);
                                      }

                                      textArea.requestFocus();
                              } //}}}
                      }
              } //}}}

}



            /**
             * <p>Title: </p>
             *
             * <p>Description: </p>
             *
             * <p>Copyright: Copyright (c) 2004</p>
             *
             * <p>Company: </p>
             * @author not attributable
             * @version 1.0
             */
            class NotValidElementDecl {
              public String elementName;
              public NotValidElementDecl(String aElementName) {
                elementName = aElementName;
              }
            }
